'''
Computes the coefficient matrices and the corresponding headers for solving the 
3S linear integer problem.
'''

import sys
import os
import csv
import math
import json

NUM_SOLVER = 12
TRAIN_DIR = '/public/xcsp_instances/new/scripts/classifiers/csp/SATzilla/features/'
OUT_DIR = '/public/xcsp_instances/new/predictions/csp/3S/sorted/'
PORTFOLIOS_FILE = '/public/xcsp_instances/new/predictions/csp/3S/sorted/portfolios_csp'
INFOTIMES_FILE = '/public/xcsp_instances/new/solvers/csp/fixed/infotimes_csp'
TIMEOUT = 1800.0
with open(INFOTIMES_FILE, 'r') as infile:
    infotimes = json.load(infile)

def solving_time_no_feat(inst, solv):
  it = infotimes[inst][solv]
  if it[0] == "sat" or it [0] == "unsat":
    # Not consider features extraction time.
    return it[1]
  else:
    return TIMEOUT

def main(args):
  portfolios = csv.reader(open(PORTFOLIOS_FILE), delimiter='|', skipinitialspace = True)
  C = TIMEOUT * 0.1
  for n_solver in range(2, NUM_SOLVER):
    row = portfolios.next()
    solvers = eval(row[1])
    for rep in range(1, 6):
      for fold in range(1, 6):
        train = 'rep' + str(rep) + '_fold' + str(fold)
	name = str(n_solver) + 'solvers_rep' + str(rep) + '_fold' + str(fold)
	print "Processing " + name
	instances = csv.reader(open(TRAIN_DIR + 'train_' + train + '.features', "rb"), delimiter=' ', skipinitialspace = True)
	header = open(OUT_DIR + str(n_solver) + '_solvers/header_' + name, 'w')
	coeff  = open(OUT_DIR + str(n_solver) + '_solvers/coeff_'  + name, 'w')	      
	insts = []
	M = n_solver
	slots = [set([]) for j in range(0, M)]
	for row in instances:
	  inst = row[0]
	  j = 0
	  ok = False
	  for solver in solvers:
	    time = solving_time_no_feat(inst, solver)
	    if time <= C:
	      if int(time) == 0:
		slots[j].add(1.0)
	      else:
		slots[j].add(math.ceil(time))
		ok = True
	    j += 1
	  if ok:
	    insts.append(inst)
	    header.write(inst + ' ')
	P = 0
	for j in range(0, M):
	  s = slots[j]
	  slots[j] = sorted(s)
	  P += len(s)
	print "Computed instances solvable in " + str(C) + " seconds and time slots for each solver"
	
	N = len(insts)
	A = [[0] * (N + P) for i in range(N + 1)]
	for i in range(0, N):
	  A[i][i] = 1
	  k = N
	  for j in range(0, M):
	    for t in slots[j]:
	      time = solving_time_no_feat(insts[i], solvers[j])
	      if time <= t:
		A[i][k] = 1
	      A[N][k] = t
	      k += 1
	      if i == 0:
		header.write(solvers[j] + '_' + str(t) + ' ')
	print "Computed coefficient matrix..."
	for i in range(0, N + 1):
	  for j in range(0, N + P):
	      coeff.write(str(A[i][j]) + ' ')
	  coeff.write('\n')
	print "Completed " + name
    header.close()
    coeff.close()

if __name__ == '__main__':
    main(sys.argv[1:])
