'''
Computes the 3S fixed schedule of solvers within a time-out of 
C seconds, using precomputed coefficient matrices and headers.
'''

import sys
import os
import csv
from gurobipy import *
from time import clock

def main(args):    
  C = 180.0
  in_path = '/home/roberto/new_experiments/3S/'
  out_file = csv.writer(open(in_path +  'schedule', "ab"), delimiter='|', skipinitialspace = True)
  num_solvers = 12
  
  for n_solver in range(2, num_solvers):
    for rep in range(1, 6):
      for fold in range(1, 6):
	start = clock()
	name = str(n_solver) + 'solvers_rep' + str(rep) + '_fold' + str(fold)
	print 'Building model ' + name + '...'
	model = Model(name)
	path = in_path + str(n_solver) + '_solvers/'
	# coeff is a coefficient matrix of size (N + 1) * M.
	coeff  = csv.reader(open(path + 'coeff_'  + name, "rb"), delimiter=" ", skipinitialspace = True)
	# header is a row of length M which contains the names of the variables.
	header = csv.reader(open(path + 'header_' + name, "rb"), delimiter=" ", skipinitialspace = True)
	
	print 'Adding binary variables to the model...'
	coeff_list = list(coeff)
	N = len(coeff_list) - 1
	header_list = list(header)[0]
	# The last element of the header (and the last column of the coefficient matrix) is actually a blank space.
	M = len(header_list) - 1
	x = [Var] * M
	for i in range(0, N):
	  # header_list[i] (i.e. the name of the instance) is too long.
	  x[i] = model.addVar(vtype = GRB.BINARY, name = 'y_' + str(i))
	for j in range(N, M):
	  x[j] = model.addVar(vtype = GRB.BINARY, name = header_list[j])
	model.update()
	
	print 'Adding contraints to the model...'
	for i in range(0, N):
	  row_vars = []
	  for j in range(N, M):
	    if int(coeff_list[i][j]) == 1:
	      row_vars.append(x[j])
	  model.addConstr(x[i] + quicksum(row_vars) >= 1)
	  
	exprs  = []
	for k in range(N, M):
	  t = coeff_list[N][k]
	  exprs.append(LinExpr(float(t), x[k]))
	model.addConstr(quicksum(exprs) <= C)
	
	print 'Adding objective function to the model...'
	for i in range(0, N):
	  exprs.append(LinExpr(float(C + 1), x[i]))
	model.setObjective(quicksum(exprs), GRB.MINIMIZE)
	
	print 'Solving the model...'
	model.optimize()
	elapsed = (clock() - start)
	print 'Objective value:', model.objVal
	print 'Schedule results:'
	sol = model.getVars()
	print 'Solution length: ' + str(len(sol))
	k = 0
	for i in range(0, N):
	  if sol[i].x == 1:
	    k += 1
	    print "Solver schedule can not solve instance: " + header_list[i]
	print "Total no. of instances that solver can't solve: " + str(k) + "/" + str(N)
	
	k = 0
	for j in range(N, M):
	  if round(sol[j].x) == 1:
	    solver_time = sol[j].varName
	    k = solver_time.index('_')
	    solver = solver_time[0 : k]
	    time = solver_time[k + 1 : len(solver_time)]
	    print "Solver schedule will run: " + solver_time
	    line = [n_solver, rep, fold, solver, time, elapsed]
	    out_file.writerow(line)
	  
if __name__ == '__main__':
    main(sys.argv[1:])
