'''
Compute ISAC predictions.
'''

import sys
import csv
import json
clusters_dir = '/public/xcsp_instances/new/scripts/classifiers/csp/ISAC/clusters/'
pred_dir     = '/public/xcsp_instances/new/predictions/csp/ISAC/'
clusters = {}
cluster_to_solver = {}
BEST_SOLVERS = '/public/xcsp_instances/new/solvers/csp/fixed/best_solvers'
INFO_TIMES = '/public/xcsp_instances/new/solvers/csp/fixed/infotimes_csp'
PORT_FILE = '/public/xcsp_instances/new/solvers/csp/fixed/portfolios_csp'
BACKUP_SOLVER = 'minisatid'
FEAT_SOLVER = 'gecode'
TIMEOUT = 1800.0
with open(INFO_TIMES, 'r') as infile:
  infotimes = json.load(infile)
with open(BEST_SOLVERS, 'r') as infile:
  bs = json.load(infile)
portfolios = csv.reader(open(PORT_FILE), delimiter='|', skipinitialspace = True)

def best_solver(cluster, solvers):
    """
    Givern a cluster of instances and a set of solvers, returns the best solver 
    (in solvers) according to no. of solved instances (in cluster), potentially using 
    resolution time for tie-breaking.
    """
    nsi = {}
    ast = {}
    for solver in solvers:
        nsi[solver] = 0
        ast[solver] = 0.0
    n = 0
    for inst in cluster:
        nosolver = True
        for solver in solvers:
            time = solver_performance(inst, solver)
            if time < TIMEOUT:
                nsi[solver] += 1
                nosolver = False
            ast[solver] += time
        if nosolver:
          n += 1
    maximum = max(nsi.values())
    # FIXME: Not consider nosolver.
    # if maximum < n:
    #   return BACKUP_SOLVER
    best_solvers = filter(lambda x : nsi[x] == maximum, nsi.keys())
    sorted_solvers = sorted(best_solvers, key = lambda x : ast[x])
    return sorted_solvers[0]
                    
def solver_performance(inst, solv):
  it = infotimes[inst][solv]
  if it[0] == "sat" or it [0] == "unsat":
    return it[1]
  else:
    return TIMEOUT

def main(args):
    solver = args[0]
    rep = args[1]
    fold = args[2]
    train = csv.reader(open(
	clusters_dir + str(solver) + '_solvers_rep' +  rep + '_fold' + fold + '_train.clusters', "rb"), 
	delimiter=" ", 
	skipinitialspace = True
    )
    for row in train:
	inst = row[0]
	cluster = row[1]
	if row[1] in clusters:
	    clusters[cluster].append(inst)
	else:
	    clusters[cluster] = [inst]
    for row in portfolios:
        if row[0] == str(solver):
            solvers = eval(row[1])
            break
    for cluster, instances in clusters.items():
	best = best_solver(instances, solvers)    
	cluster_to_solver[cluster] = best

    #print cluster_to_solver, len(cluster_to_solver)     

    test = csv.reader(open(
	clusters_dir + str(solver) + '_solvers_rep' + rep + '_fold' + fold + '_test.clusters', "rb"), 
	delimiter=" ", 
	skipinitialspace = True
    )
    pred_file = csv.writer(
	open(pred_dir + 'pred_' + str(solver) + 'solvers', "ab"), 
	delimiter='|', 
	skipinitialspace = True
    )
    for row in test:
	inst = row[0]
	cluster = row[1]
	pred = cluster_to_solver[cluster]
        it = infotimes[inst][pred]
        if pred == FEAT_SOLVER:
          time = float(bs[inst]['feat']) + it[1] - float(bs[inst]['mzn2fzn'])
        else:
          time = float(bs[inst]['feat']) + it[1]
        if it[0] != "sat" and it [0] != "unsat":
          if it[0] != "timeout":
            itb = infotimes[inst][BACKUP_SOLVER]
            time += itb[1]
            if time < TIMEOUT and (itb[0] == "sat" or itb[0] == "unsat"):
              succ = 1
            else:
              time = TIMEOUT
              succ = 0
          else:
            time = TIMEOUT
            succ = 0
        else:
          if time < TIMEOUT:
            succ = 1
          else:
            time = TIMEOUT
            succ = 0
	line = [inst, pred, succ, time, rep, fold]
	pred_file.writerow(line)

if __name__ == '__main__':
    main(sys.argv[1:])
