'''
Compute statistics for VBS and the constituent solvers.
'''
import sys
import os
import csv
import json

solvers = {
  'bprolog', 'fzn2smt', 'g12cpx', 'g12fd', 'g12lazyfd', 'g12mip', 'gecode', 
  'izplus', 'jacop', 'minisatid', 'mistral', 'ortools'
}
BACKUP_SOLVER = 'minisatid'
TIMEOUT = 1800.0
infotimes_file = '/public/xcsp_instances/new/solvers/csp/fixed/infotimes_csp'
portfolios_file = '/public/xcsp_instances/new/solvers/csp/fixed/portfolios_csp'

def compute_predictions(solver, infotimes):
  # Total time to compute the instances.
  total_time = 0.0
  # Number of instances.
  counter = 0
  solved_instances = 0
  for item in infotimes.items():
    counter += 1
    instance = item[0]
    it = item[1][solver]
    if it[0] == "sat" or it [0] == "unsat":
      solved_instances += 1
      total_time += it[1]
    else:
      total_time += TIMEOUT
  avg = str(total_time / counter)
  psi = str(100 * float(solved_instances) / counter)
  return solver + '|' + avg + '|' + psi

def compute_predictions_vbs(portfolio, infotimes):
  total_time = 0.0
  solved_instances = 0
  counter = 0
  for item in infotimes.items():
    counter += 1
    instance = item[0]
    it = item[1]
    best = filter(lambda s: it[s][0] == "sat" or it[s][0] == "unsat", portfolio)
    if len(best) > 0:
      solved_instances += 1
      best_solver = min(best, key = lambda s: it[s][1])
      total_time += it[best_solver][1]
    else:
      total_time += TIMEOUT
  avg = str(total_time / counter)
  psi = str(100 * float(solved_instances) / counter)
  return 'vbs_' + str(len(portfolio)) + '|' + avg + '|' + psi

        
def main(args):
  with open(infotimes_file, 'r') as infile:
    infotimes = json.load(infile)
  for solver in solvers:
    cp = compute_predictions(solver, infotimes)
    print cp

  portfolios_reader = csv.reader(open(portfolios_file,  'rb'),  delimiter = '|')
  for row in portfolios_reader:
    cp = compute_predictions_vbs(eval(row[1]), infotimes)
    print cp

if __name__ == '__main__':
    main(sys.argv[1:])
