'''
Compute statistics for off-the-shelf classifiers (called by compute_statistics_ots).
'''
import sys
import os
import json

BACKUP_SOLVER = 'minisatid'
FEAT_SOLVER = 'gecode'
TIMEOUT = 1800.0
infotimes_file = '/public/xcsp_instances/new/solvers/csp/fixed/infotimes_csp'
bestsolvers_file = '/public/xcsp_instances/new/solvers/csp/fixed/best_solvers'

def compute_predictions(pred_path, infotimes, best_solvers):
  # Total time to compute the instances.
  total_time = 0.0
  # Number of instances.
  counter = 0
  solved_instances = 0
  dir_list = sorted(os.listdir(pred_path))
    
  for file_name in dir_list:
    f = open(pred_path + "/" + file_name)
    # Skip the first line.
    f.readline() 
    line = f.readline()
    while line:
      row = line.split()
      if len(row) > 0:
        counter += 1
        # Get the predicted solver and the instance name.
        solver = row[2].split(':')[1]
        instance = row[-1][1:-1]
        if solver == 'minisati':
          solver = 'minisatid'
        elif solver == 'g12lazyf':
          solver = 'g12lazyfd'
        elif solver == 'nosolver':
          solver = BACKUP_SOLVER
        it = infotimes[instance][solver]
        if solver == FEAT_SOLVER:
          time = float(best_solvers[instance]['feat']) + it[1] - float(best_solvers[instance]['mzn2fzn'])
        else:
          time = float(best_solvers[instance]['feat']) + it[1]
        if it[0] != "sat" and it [0] != "unsat":
          if it[0] != "timeout":
            itb = infotimes[instance][BACKUP_SOLVER]
            time += itb[1]
            if time < TIMEOUT and (itb[0] == "sat" or itb[0] == "unsat"):
              solved_instances += 1
            else:
              time = TIMEOUT
          else:
            time = TIMEOUT
        else:
          if time < TIMEOUT:
            solved_instances += 1
          else:
            time = TIMEOUT
        total_time += time
      line = f.readline()
  avg = str(total_time / counter)
  psi = str(100 * float(solved_instances) / counter)
  return avg + '|' + psi
        
def main(args):
  with open(infotimes_file, 'r') as infile:
    infotimes = json.load(infile)
  with open(bestsolvers_file, 'r') as infile:
    best_solvers = json.load(infile)
  cp = compute_predictions(args[0], infotimes, best_solvers)
  print cp

if __name__ == '__main__':
    main(sys.argv[1:])
